/*
    FFT Display
    Copyright © 2010-2023 Harry Whitfield

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public
    License along with this program; if not, write to the Free
    Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
    MA  02110-1301  USA

    FFT Display - browser version 1.1
    26 February, 2023
    Copyright © 2010-2023 Harry Whitfield
    mailto:g6auc@arrl.net
*/

/*jslint browser */

/*global newImage, newText, newInput, newTextArea, newCanvas, newButton, newSelector,
    addToMenu, moveObj, fScale, waveTitle, openBarChart, testFFT, waveformPref,
    window, alert
*/

/*property
    backgroundColor, border, clientX, close, forEach, hOffset, innerWidth,
    maxWidth, onclick, onmousedown, onmouseleave, onmousemove, onmouseout,
    onmouseover, onmouseup, ontouchcancel, ontouchend, ontouchmove,
    ontouchstart, open, preventDefault, round, selectedIndex, size, src, style,
    targetTouches, title, tracking, value, width
*/

/////////////////////////////// Start of the FFT Display /////////////////////////////////

// newImage(hOffset, vOffset, width, height, src, zOrder, opacity)
// newText(hOffset, vOffset, width, height, value, zOrder, style)
// newTextArea(hOffset, vOffset, cols, rows, value, zOrder, style)
// newInput(hOffset, vOffset, width, height, value, zOrder, style)
// newCanvas(hOffset, vOffset, width, height, src, zOrder, opacity, hRegP, vRegP)
// newSelector(hOffset, vOffset, width, height, src, zOrder, opacity)

var base = "Resources/images/";

var leftbg = newImage(0, 0, 30, 79, base + "leftbg.png", 1, 1.0);
var midbg = newImage(30, 0, 540, 79, base + "midbg.png", 1, 1.0);
var rightbg = newImage(570, 0, 30, 79, base + "rightbg.png", 1, 1.0);

var dragbox = newImage(567, 61, 30, 15, base + "dragbox.png", 2, 2);
dragbox.title = "Drag me to change the widget's width.";
dragbox.tracking = "rectangle";

base = "Resources/buttons/";

var redButton = newImage(8, 5, 21, 14, base + "red.png", 2);
var amberButton = newImage(29, 5, 21, 14, base + "amber.png", 2);
var greenButton = newImage(50, 5, 14, 14, base + "green.png", 2);

redButton.title = "Click to close the Widget.";
amberButton.title = "Clear all.";
greenButton.title = "Click to toggle the window size.";
/*
var helpButton = newImage(330, 57, 30, 22, base + "help.png", 2);
helpButton.title = "Displays information about this program.";
*/
var style = "background-color:transparent;border:none;text-align:center;font-size:12px;";
var title = newText(70, 0, 65, 18, "FFT Display", 2, style);

base = "Resources/images/";

var cmdHOff = 300;

var comBgLeft = newImage(cmdHOff, 34, 20, 22, base + "comBgLeft.png", 2);
var comBgMiddle = newImage(cmdHOff + 20, 34, 250, 22, base + "comBgMidd.png", 2);
var comBgRight = newImage(cmdHOff + 270, 34, 20, 22, base + "comBgRight.png", 2);

style = "background-color:#84e67c;font-family:Arial;font-size:14px";    // set fScale to 6.0

var report = newTextArea(-1, 337, 100, 14, "", 1, style);   // declared elsewhere
report.title = "Report Data.";
report.style.border = "1px solid black";
report.style.maxWidth = "564px";
report.value = "";

var barChart = newCanvas(1, 80, 568, 256, "", 1, 1.0);  // declared elsewhere
barChart.style.backgroundColor = "#f5f8b8";
barChart.style.border = "1px solid black";
/*
var buttons = newImage(1, 570, 568, 48, base + "buttons.png", 2);
buttons.style.border = "1px solid black";
var cancel = newImage(396, 581, 72, 26, base + "cancel.png", 2);
var save = newImage(472, 580, 72, 26, base + "save.png", 2);
*/
style = "background-color:transparent;border:none;font-family:'Arial', 'Courier New';font-weight: normal;font-size:14px";

var command = newInput(cmdHOff + 10, 35, 120, 17, "", 3, style);
command.title = "Enter a javascript statement in this box, then press RETURN or ENTER.";
command.title += " Use the SHIFT key when pressing RETURN or ENTER to clear the box after the statement is executed.";

function clearData() {
    command.value = "";
    report.value = "";
    openBarChart(barChart);
}

function button1Action() {
    openBarChart(barChart);
}
function button2Action() {
    report.value = "";
}
function button3Action() {
    clearData();
}
function button4Action() {
    testFFT(waveformPref);
}
function button5Action() {
    window.open("Help.html");
}

var button1 = newButton(10, 34, 48, 24, base + "open.png", base + "openDn.png", 2, 1.0, button1Action);
button1.title = "Clear the graph.";

var button2 = newButton(68, 34, 48, 24, base + "open.png", base + "openDn.png", 2, 1.0, button2Action);
button2.title = "Clear the report.";

var button3 = newButton(126, 34, 48, 24, base + "open.png", base + "openDn.png", 2, 1.0, button3Action);
button3.title = "Clear all.";

var button4 = newButton(184, 34, 48, 24, base + "speed.png", base + "speedDn.png", 2, 1.0, button4Action);
button4.title = "(Re)calculate the FFT using the currently selected example waveform.";

var button5 = newButton(242, 34, 48, 24, base + "star.png", base + "starDn.png", 2, 1.0, button5Action);
button5.title = "Displays information about this program.";

style = "background-color:transparent;border:none;font-family:Arial;font-weight:normal;font-size:9px;text-align:left";

var Label1 = newText(6, 57, 75, 16, "Clear Graph", 2, style);
var Label2 = newText(63, 57, 75, 16, "Clear Report", 2, style);
var Label3 = newText(130, 57, 75, 16, "Clear All", 2, style);
var Label4 = newText(185, 57, 60, 16, "Calculate", 2, style);
var Label5 = newText(241, 57, 60, 16, "Open Help", 2, style);

var commandLabel = newText(385, 57, 100, 16, "Command", 2, style);

var minWidth = 572; // was 600
var maxWidth = minWidth;
var curWidth = minWidth;

if (window.innerWidth > minWidth) {
	maxWidth = window.innerWidth;
}

//////////////////////////////////// Menus Customized ////////////////////////////////////

var multiplierValues = [1e0, 1e3, 1e6, 1e9, 1e12];

var MenuArray = [
    waveTitle,
    ["0.1", "0.2", "0.4", "0.5", "0.8", "1.0", "2.0", "4.0", "5.0", "8.0", "10.0"],
    ["unity", "k", "M", "G", "T"],
    ["Units", "Hz", "rad/s"]
];

var MenuTitleArray = [
    "Choose an example waveform and calculate it's FFT.",
    "Choose the scale factor." +
    " Note that this also recalculates the FFT using the currently selected example waveform.",
    "Choose the scale multiplier. Used only when it is omitted from a makeFFT command.",
    "Choose the units to be displayed for frequency on the graph and in the report." +
    " Used only when it is omitted from a makeFFT command." +
    " Note that this is used for display purposes only and has no effect on the calculation."
];

var MenuSelectedIndex = [4, 5, 0, 0];

var Menu = [];

Menu[0] = newSelector(150, 1, 110, 20, "", 2);
Menu[1] = newSelector(365, 1, 110, 20, "", 2);
Menu[2] = newSelector(430, 1, 110, 20, "", 2);
Menu[3] = newSelector(500, 1, 110, 20, "", 2);

(function () {
    MenuArray.forEach(function (ele, i) {
        addToMenu(Menu[i], ele);
        Menu[i].selectedIndex = MenuSelectedIndex[i];
        Menu[i].title = MenuTitleArray[i];
    });
}());

//////////////////////////////////////////////////////////////////////////////////////////

var print = function (s) {
    alert(s);
};
var dragEnabled = false;

function setUp(width) {
    if (width < minWidth) {
        width = minWidth;
    }
    if (width > maxWidth) {
        width = maxWidth;
    }

    midbg.width = width - 60;
    rightbg.hOffset = width - 30;
    moveObj(rightbg, width - 30, 0);

    comBgMiddle.width = width - cmdHOff - 50;
    comBgRight.hOffset = width - 30;
    moveObj(comBgRight, width - 30, 34);

    dragbox.hOffset = width - 33;
    moveObj(dragbox, width - 33, 61);

//  title.hOffset = width / 2 - 50;
//  moveObj(title, width / 2 - 50, 3);

//  commandLabel.hOffset = cmdHOff - 50 + (width - cmdHOff) / 2;
//  moveObj(commandLabel, cmdHOff - 50 + (width - cmdHOff) / 2, 57);

//  report.width = width;
//  report.cols = Math.round(width / 6.0);  // was fScale = 6.67
//  report.style.maxWidth = String(width - 8) + "px";

//  helpButton.hOffset = width - 38;
//  moveObj(helpButton, width - 38, 3);

    command.width = width - cmdHOff - 38;
    command.size = Math.round((width - cmdHOff - 38) / 6.0);    // was fScale = 6.67
    command.style.maxWidth = String(width - cmdHOff - 38) + "px";

    return width;
}

//////////////////////////////////////////////////////////////////////////////////////////

dragbox.onmousedown = function (evt) {
	evt.preventDefault();
	dragEnabled = true;
};
dragbox.onmousemove = function (evt) {
	evt.preventDefault();
    if (dragEnabled) {
        moveObj(dragbox, evt.clientX, 61);
        curWidth = setUp(evt.clientX + 15);
    }
};
dragbox.onmouseup = function (evt) {
	evt.preventDefault();
	dragEnabled = false;
};
dragbox.onmouseleave = function (evt) {
	evt.preventDefault();
	dragEnabled = false;
};

dragbox.ontouchstart = function (evt) {
    evt.preventDefault();
	dragEnabled = true;
};
dragbox.ontouchmove = function (evt) {
	evt.preventDefault();
    if (dragEnabled) {
        moveObj(dragbox, evt.targetTouches[0].clientX, 61);
        curWidth = setUp(evt.targetTouches[0].clientX + 15);
    }
};
dragbox.ontouchend = function (evt) {
    evt.preventDefault();
	dragEnabled = false;
};
dragbox.ontouchcancel = function (evt) {
    evt.preventDefault();
	dragEnabled = false;
};

//////////////////////////////////////////////////////////////////////////////////////////

function normalButtons() {
    var base1 = "Resources/buttons/";

    redButton.src = base1 + "red.png";
    amberButton.src = base1 + "amber.png";
    greenButton.src = base1 + "green.png";
}

function activeButtons() {
    var base1 = "Resources/buttons/";

    redButton.src = base1 + "redCross.png";
    amberButton.src = base1 + "amberMinus.png";
    greenButton.src = base1 + "greenPlus.png";
}

redButton.onmouseover = function () {
    activeButtons();
};

redButton.onmouseout = function () {
    normalButtons();
};

amberButton.onmouseover = function () {
    activeButtons();
};

amberButton.onmouseout = function () {
    normalButtons();
};

greenButton.onmouseover = function () {
    activeButtons();
};

greenButton.onmouseout = function () {
    normalButtons();
};

redButton.onclick = function () {
    window.close();
};

amberButton.onclick = function () {
    clearData();
};

greenButton.onclick = function () {
    dragEnabled = false;

    if (curWidth === minWidth) {
        curWidth = setUp(maxWidth);
    } else {
        curWidth = setUp(minWidth);
    }
};
/*
helpButton.onmousedown = function () { this.style.opacity = "0.5"; };
helpButton.onmouseup = function () { this.style.opacity = "1.0"; window.open("Help.html"); };
*/
setUp(minWidth);
openBarChart(barChart);

